/*
****************************************************************************
* Copyright (C) 2008 - 2015 Bosch Sensortec GmbH
*
* bmp180.c
* Date: 2015/03/27
* Revision: 2.0.3 $
*
* Usage: Sensor Driver file for BMP180
*
****************************************************************************
* License:
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*   Redistributions of source code must retain the above copyright
*   notice, this list of conditions and the following disclaimer.
*
*   Redistributions in binary form must reproduce the above copyright
*   notice, this list of conditions and the following disclaimer in the
*   documentation and/or other materials provided with the distribution.
*
*   Neither the name of the copyright holder nor the names of the
*   contributors may be used to endorse or promote products derived from
*   this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER
* OR CONTRIBUTORS BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
* OR CONSEQUENTIAL DAMAGES(INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
* ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE
*
* The information provided is believed to be accurate and reliable.
* The copyright holder assumes no responsibility
* for the consequences of use
* of such information nor for any infringement of patents or
* other rights of third parties which may result from its use.
* No license is granted by implication or otherwise under any patent or
* patent rights of the copyright holder.
**************************************************************************/
#include "bmp180.h"
static struct bmp180_t *p_bmp180;

#include "cmsis_os2.h"
#include "rtx_os.h"
#include "arm_math.h"

/*!
 *	@brief This function is used for initialize
 *	the bus read and bus write functions
 *  and assign the chip id and I2C address of the BMP180
 *	chip id is read in the register 0xD0 bit from 0 to 7
 *
 *	 @param bmp180 structure pointer.
 *
 *	@note While changing the parameter of the bmp180_t
 *	@note consider the following point:
 *	Changing the reference value of the parameter
 *	will changes the local copy or local reference
 *	make sure your changes will not
 *	affect the reference value of the parameter
 *	(Better case don't change the reference value of the parameter)
 *
 *
 *
 *
 *	@return results of bus communication function
 *	@retval 0 -> Success
 *	@retval -1 -> Error
 *
 *
*/
BMP180_RETURN_FUNCTION_TYPE bmp180_init(struct bmp180_t *bmp180)
{
	/* used to return the bus communication results*/
	BMP180_RETURN_FUNCTION_TYPE v_com_rslt_s8 = E_BMP_COMM_RES;
	u8 v_data_u8 = BMP180_INIT_VALUE;
	/* assign BMP180 ptr */
	p_bmp180 = bmp180;
	/* read Chip Id */
	v_com_rslt_s8 = p_bmp180->BMP180_BUS_READ_FUNC(p_bmp180->dev_addr, BMP180_CHIP_ID__REG,&v_data_u8, BMP180_GEN_READ_WRITE_DATA_LENGTH);
	p_bmp180->chip_id = BMP180_GET_BITSLICE(v_data_u8, BMP180_CHIP_ID);
	p_bmp180->number_of_samples = BMP180_INITIALIZE_NUMBER_OF_SAMPLES_U8X;
	p_bmp180->oversamp_setting = BMP180_INITIALIZE_OVERSAMP_SETTING_U8X;
	p_bmp180->sw_oversamp = BMP180_INITIALIZE_SW_OVERSAMP_U8X;
	v_com_rslt_s8 += p_bmp180->BMP180_BUS_READ_FUNC(p_bmp180->dev_addr, BMP180_VERSION_REG,&v_data_u8, BMP180_GEN_READ_WRITE_DATA_LENGTH);
	/* read Version reg */

	p_bmp180->ml_version = BMP180_GET_BITSLICE(
	v_data_u8, BMP180_ML_VERSION);/* get ML Version */
	p_bmp180->al_version = BMP180_GET_BITSLICE(
	v_data_u8, BMP180_AL_VERSION); /* get AL Version */
	v_com_rslt_s8 += bmp180_get_calib_param();

	return v_com_rslt_s8;
}
/*!
 *	@brief this function used for read the calibration
 *	parameter from the register
 *
 *	Parameter  |  MSB    |  LSB    |  bit
 * ------------|---------|---------|-----------
 *		AC1    |  0xAA   | 0xAB    | 0 to 7
 *		AC2    |  0xAC   | 0xAD    | 0 to 7
 *		AC3    |  0xAE   | 0xAF    | 0 to 7
 *		AC4    |  0xB0   | 0xB1    | 0 to 7
 *		AC5    |  0xB2   | 0xB3    | 0 to 7
 *		AC6    |  0xB4   | 0xB5    | 0 to 7
 *		B1     |  0xB6   | 0xB7    | 0 to 7
 *		B2     |  0xB8   | 0xB9    | 0 to 7
 *		MB     |  0xBA   | 0xBB    | 0 to 7
 *		MC     |  0xBC   | 0xBD    | 0 to 7
 *		MD     | 0xBE    | 0xBF    | 0 to 7
 *
 *
 *	@return results of bus communication function
 *	@retval 0 -> Success
 *	@retval -1 -> Error
 *
 *
*/
BMP180_RETURN_FUNCTION_TYPE bmp180_get_calib_param(void)
{
	/* used to return the bus communication results*/
	BMP180_RETURN_FUNCTION_TYPE v_com_rslt_s8 = E_BMP_COMM_RES;
	/* Array holding the calibration informations*/
	u8 a_data_u8r[BMP180_CALIB_DATA_SIZE] = {
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE};
	/* read calibration data*/
	v_com_rslt_s8 = p_bmp180->BMP180_BUS_READ_FUNC(
	p_bmp180->dev_addr, BMP180_PROM_START__ADDR,
	a_data_u8r, BMP180_PROM_DATA__LEN);

	/*parameters AC1-AC6*/
	p_bmp180->calib_param.ac1 =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_AC1_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC1_LSB]);
	p_bmp180->calib_param.ac2 =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_AC2_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC2_LSB]);
	p_bmp180->calib_param.ac3 =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_AC3_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC3_LSB]);
	p_bmp180->calib_param.ac4 =
	(u16)((((u32)((u8)a_data_u8r[BMP180_CALIB_PARAM_AC4_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC4_LSB]);
	p_bmp180->calib_param.ac5 =
	(u16)((((u32)((u8)a_data_u8r[BMP180_CALIB_PARAM_AC5_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC5_LSB]);
	p_bmp180->calib_param.ac6 =
	(u16)((((u32)((u8)a_data_u8r[BMP180_CALIB_PARAM_AC6_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_AC6_LSB]);

	/*parameters B1,B2*/
	p_bmp180->calib_param.b1 =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_B1_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS) |
	a_data_u8r[BMP180_CALIB_PARAM_B1_LSB]);
	p_bmp180->calib_param.b2 =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_B2_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_B2_LSB]);

	/*parameters MB,MC,MD*/
	p_bmp180->calib_param.mb =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_MB_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_MB_LSB]);
	p_bmp180->calib_param.mc =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_MC_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_MC_LSB]);
	p_bmp180->calib_param.md =
	(s16)((((s32)((s8)a_data_u8r[BMP180_CALIB_PARAM_MD_MSB]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| a_data_u8r[BMP180_CALIB_PARAM_MD_LSB]);
	return v_com_rslt_s8;
}
/*!
 *	@brief this API is used to calculate the true
 *	temperature using the uncompensated temperature(ut)
 *	@note For reading the ut data refer : bmp180_read_ut()
 *
 *	@param v_uncomp_temperature_u32:
 *	the value of uncompensated temperature
 *
 *	@return Return the temperature in steps of 0.1 deg Celsius
 *
 *
*/
s16 bmp180_get_temperature(u32 v_uncomp_temperature_u32)
{
	s16 v_temperature_s16 = BMP180_INIT_VALUE;
	s32 v_x1_s32, v_x2_s32 = BMP180_INIT_VALUE;
	/* calculate temperature*/
	v_x1_s32 = (((s32) v_uncomp_temperature_u32 -
	(s32) p_bmp180->calib_param.ac6) *
	(s32) p_bmp180->calib_param.ac5)
	>> BMP180_SHIFT_BIT_POSITION_BY_15_BITS;
	if (v_x1_s32 == BMP180_CHECK_DIVISOR && p_bmp180->calib_param.md
	== BMP180_CHECK_DIVISOR)
		return BMP180_INVALID_DATA;
	/* executed only the divisor is not zero*/
	v_x2_s32 = ((s32) p_bmp180->calib_param.mc
	<< BMP180_SHIFT_BIT_POSITION_BY_11_BITS) /
	(v_x1_s32 + p_bmp180->calib_param.md);

	p_bmp180->param_b5 = v_x1_s32 + v_x2_s32;
	v_temperature_s16 = ((p_bmp180->param_b5 +
	BMP180_CALCULATE_TRUE_TEMPERATURE)
	>> BMP180_SHIFT_BIT_POSITION_BY_04_BITS);

	return v_temperature_s16;
}
/*!
 *	@brief this API is used to calculate the true
 *	pressure using the uncompensated pressure(up)
 *	@note For reading the up data refer : bmp180_read_up()
 *
 *	@param v_uncomp_pressure_u32: the value of uncompensated pressure
 *
 *	@return Return the value of pressure in steps of 1.0 Pa
 *
*/
s32 bmp180_get_pressure(u32 v_uncomp_pressure_u32)
{
	s32 v_pressure_s32, v_x1_s32, v_x2_s32,
	v_x3_s32, v_b3_s32, v_b6_s32 = BMP180_INIT_VALUE;
	u32 v_b4_u32, v_b7_u32 = BMP180_INIT_VALUE;

		v_b6_s32 = p_bmp180->param_b5 - 4000;
	/*****calculate B3************/
	v_x1_s32 = (v_b6_s32*v_b6_s32)
	>> BMP180_SHIFT_BIT_POSITION_BY_12_BITS;
	v_x1_s32 *= p_bmp180->calib_param.b2;
	v_x1_s32 >>= BMP180_SHIFT_BIT_POSITION_BY_11_BITS;

	v_x2_s32 = (p_bmp180->calib_param.ac2*v_b6_s32);
	v_x2_s32 >>= BMP180_SHIFT_BIT_POSITION_BY_11_BITS;

	v_x3_s32 = v_x1_s32 + v_x2_s32;
	v_b3_s32 = (((((s32)p_bmp180->calib_param.ac1)*4 + v_x3_s32) <<
	p_bmp180->oversamp_setting) + 2)
	>> BMP180_SHIFT_BIT_POSITION_BY_02_BITS;

	/*****calculate B4************/
	v_x1_s32 = (p_bmp180->calib_param.ac3 * v_b6_s32)
	>> BMP180_SHIFT_BIT_POSITION_BY_13_BITS;
	v_x2_s32 = (p_bmp180->calib_param.b1 *
	((v_b6_s32 * v_b6_s32) >> BMP180_SHIFT_BIT_POSITION_BY_12_BITS))
	>> BMP180_SHIFT_BIT_POSITION_BY_16_BITS;
	v_x3_s32 = ((v_x1_s32 + v_x2_s32) + 2)
	>> BMP180_SHIFT_BIT_POSITION_BY_02_BITS;
	v_b4_u32 = (p_bmp180->calib_param.ac4 * (u32)
	(v_x3_s32 + 32768)) >> BMP180_SHIFT_BIT_POSITION_BY_15_BITS;

	v_b7_u32 = ((u32)(v_uncomp_pressure_u32 - v_b3_s32) *
	(50000 >> p_bmp180->oversamp_setting));
	if (v_b7_u32 < 0x80000000) {
		if (v_b4_u32 != BMP180_CHECK_DIVISOR)
			v_pressure_s32 =
			(v_b7_u32
			<< BMP180_SHIFT_BIT_POSITION_BY_01_BIT) / v_b4_u32;
		 else
			return BMP180_INVALID_DATA;
	} else {
		if (v_b4_u32 != BMP180_CHECK_DIVISOR)
			v_pressure_s32 = (v_b7_u32 / v_b4_u32)
			<< BMP180_SHIFT_BIT_POSITION_BY_01_BIT;
		else
			return BMP180_INVALID_DATA;
	}

		v_x1_s32 = v_pressure_s32
		>> BMP180_SHIFT_BIT_POSITION_BY_08_BITS;
		v_x1_s32 *= v_x1_s32;
		v_x1_s32 =
		(v_x1_s32 * BMP180_PARAM_MG)
		>> BMP180_SHIFT_BIT_POSITION_BY_16_BITS;
		v_x2_s32 = (v_pressure_s32 * BMP180_PARAM_MH)
		>> BMP180_SHIFT_BIT_POSITION_BY_16_BITS;
		/*pressure in Pa*/
		v_pressure_s32 += (v_x1_s32 + v_x2_s32 + BMP180_PARAM_MI)
		>> BMP180_SHIFT_BIT_POSITION_BY_04_BITS;
	return v_pressure_s32;
}
/*!
 *	@brief this API is used to read the
 *	uncompensated temperature(ut) from the register
 *	@note 0xF6(MSB) bit from 0 to 7
 *	@note 0xF7(LSB) bit from 0 to 7
 *
 *
 *	@return results of bus communication function
 *	@retval 0 -> Success
 *	@retval -1 -> Error
 *
 *
*/
u16 bmp180_get_uncomp_temperature(void)
{
	u16 v_ut_u16 = BMP180_INIT_VALUE;
	/* Array holding the temperature LSB and MSB data*/
	u8 v_data_u8[BMP180_TEMPERATURE_DATA_BYTES] = {
	BMP180_INIT_VALUE, BMP180_INIT_VALUE};
	u8 v_ctrl_reg_data_u8 = BMP180_INIT_VALUE;
	/* used to return the bus communication results*/
	BMP180_RETURN_FUNCTION_TYPE v_com_rslt_s8 = E_BMP_COMM_RES;
	u8 v_wait_time_u8 = BMP180_INIT_VALUE;

	v_ctrl_reg_data_u8 = BMP180_T_MEASURE;
	v_wait_time_u8 = BMP180_TEMP_CONVERSION_TIME;

	v_com_rslt_s8 = p_bmp180->BMP180_BUS_WRITE_FUNC(p_bmp180->dev_addr,
	BMP180_CTRL_MEAS_REG,
	&v_ctrl_reg_data_u8, BMP180_GEN_READ_WRITE_DATA_LENGTH);
	p_bmp180->delay_msec(v_wait_time_u8);
	v_com_rslt_s8 +=
	p_bmp180->BMP180_BUS_READ_FUNC(p_bmp180->dev_addr,
	BMP180_ADC_OUT_MSB_REG, v_data_u8,
	BMP180_TEMPERATURE_DATA_LENGTH);
	v_ut_u16 = (u16)((((s32)
	((s8)v_data_u8[BMP180_TEMPERATURE_MSB_DATA]))
	<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS)
	| (v_data_u8[BMP180_TEMPERATURE_LSB_DATA]));
	return v_ut_u16;
}
/*!
 *	@brief this API is used to read the
 *	uncompensated pressure(up) from the register
 *	@note 0xF6(MSB) bit from 0 to 7
 *	@note 0xF7(LSB) bit from 0 to 7
 *	@note 0xF8(LSB) bit from 3 to 7
 *
 *
 *	@return results of bus communication function
 *	@retval 0 -> Success
 *	@retval -1 -> Error
 *
*/
u32 bmp180_get_uncomp_pressure(void)
{
	/*j included for loop*/
	u8 v_j_u8 = BMP180_INIT_VALUE;
	u32 v_up_u32 = BMP180_INIT_VALUE;
	/*get the calculated pressure data*/
	u32 v_sum_u32 = BMP180_INIT_VALUE;
	u8 v_data_u8[BMP180_PRESSURE_DATA_BYTES] = {
	BMP180_INIT_VALUE,
	BMP180_INIT_VALUE, BMP180_INIT_VALUE};
	u8 v_ctrl_reg_data_u8 = BMP180_INIT_VALUE;
	/* used to return the bus communication results*/
	BMP180_RETURN_FUNCTION_TYPE v_com_rslt_s8 = E_BMP_COMM_RES;

	if (p_bmp180->sw_oversamp == BMP180_SW_OVERSAMP_U8X &&
	p_bmp180->oversamp_setting == BMP180_OVERSAMP_SETTING_U8X) {
		for (v_j_u8 = BMP180_INIT_VALUE;
		v_j_u8 < BMP180_DATA_MEASURE; v_j_u8++) {
			/* 3 times getting pressure data*/
			v_ctrl_reg_data_u8 = BMP180_P_MEASURE +
			(p_bmp180->oversamp_setting
			<< BMP180_SHIFT_BIT_POSITION_BY_06_BITS);
			v_com_rslt_s8 = p_bmp180->BMP180_BUS_WRITE_FUNC(
			p_bmp180->dev_addr,
			BMP180_CTRL_MEAS_REG,
			&v_ctrl_reg_data_u8, BMP180_GEN_READ_WRITE_DATA_LENGTH);
			p_bmp180->delay_msec(BMP180_2MS_DELAY_U8X + (
			BMP180_3MS_DELAY_U8X <<
			(p_bmp180->oversamp_setting)));
			v_com_rslt_s8 +=
			p_bmp180->BMP180_BUS_READ_FUNC(
			p_bmp180->dev_addr,
			BMP180_ADC_OUT_MSB_REG,
			v_data_u8, BMP180_PRESSURE_DATA_LENGTH);
			v_sum_u32 = (u32)((((u32)
			v_data_u8[BMP180_PRESSURE_MSB_DATA]
			<< BMP180_SHIFT_BIT_POSITION_BY_16_BITS) |
			((u32) v_data_u8[BMP180_PRESSURE_LSB_DATA]
			<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS) |
			(u32) v_data_u8[BMP180_PRESSURE_XLSB_DATA]) >>
			(BMP180_CALCULATE_TRUE_PRESSURE -
			p_bmp180->oversamp_setting));
			p_bmp180->number_of_samples =
			BMP180_INITIALIZE_NUMBER_OF_SAMPLES_U8X;

			v_up_u32 = v_up_u32 + v_sum_u32;
			/*add up with dummy var*/
		}
		v_up_u32 = v_up_u32 / BMP180_AVERAGE_U8X; /*averaging*/
	} else {
		if (p_bmp180->sw_oversamp ==
		BMP180_INITIALIZE_SW_OVERSAMP_U8X) {
			v_ctrl_reg_data_u8 = BMP180_P_MEASURE +
			(p_bmp180->oversamp_setting
			<< BMP180_SHIFT_BIT_POSITION_BY_06_BITS);
			v_com_rslt_s8 = p_bmp180->BMP180_BUS_WRITE_FUNC(
			p_bmp180->dev_addr, BMP180_CTRL_MEAS_REG,
			&v_ctrl_reg_data_u8, BMP180_GEN_READ_WRITE_DATA_LENGTH);
			p_bmp180->delay_msec(BMP180_2MS_DELAY_U8X
			+ (BMP180_3MS_DELAY_U8X <<
			(p_bmp180->oversamp_setting)));
			v_com_rslt_s8 += p_bmp180->BMP180_BUS_READ_FUNC(
			p_bmp180->dev_addr,
			BMP180_ADC_OUT_MSB_REG,
			v_data_u8, BMP180_PRESSURE_DATA_LENGTH);
			v_up_u32 = (u32)((((u32)
			v_data_u8[BMP180_PRESSURE_MSB_DATA]
			<< BMP180_SHIFT_BIT_POSITION_BY_16_BITS) | (
			(u32) v_data_u8[BMP180_PRESSURE_LSB_DATA]
			<< BMP180_SHIFT_BIT_POSITION_BY_08_BITS) |
			(u32) v_data_u8[BMP180_PRESSURE_XLSB_DATA]) >>
			(BMP180_CALCULATE_TRUE_PRESSURE -
			p_bmp180->oversamp_setting));
			p_bmp180->number_of_samples =
			BMP180_INITIALIZE_NUMBER_OF_SAMPLES_U8X;
		}

	}

	return v_up_u32;
}

#include "bmp180.h"
#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_i2c.h"

/*----------------------------------------------------------------------------
  * The following functions are used for reading and writing of
  * sensor data using I2C or SPI communication
----------------------------------------------------------------------------*/
//#define BMP180_API
#define	C_BMP180_ONE_U8X	-1

#ifdef BMP180_API
/*	\Brief: The function is used as I2C bus read
 *	\Return : Status of the I2C read
 *	\param dev_addr : The device address of the sensor
 *	\param reg_addr : Address of the first register, will data is going to be read
 *	\param reg_data : This data read from the sensor, which is hold in an array
 *	\param cnt : The no of byte of data to be read
 */
s8 BMP180_I2C_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);
/*	\Brief: The function is used as SPI bus write
 *	\Return : Status of the SPI write
 *	\param dev_addr : The device address of the sensor
 *	\param reg_addr : Address of the first register, will data is going to be written
 *	\param reg_data : It is a value hold in the array,
 *		will be used for write the value into the register
 *	\param cnt : The no of byte of data to be write
 */
s8 BMP180_I2C_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);
/*
 * \Brief: I2C init routine
*/
s8 I2C_routine(void);
#endif
/********************End of I2C function declarations***********************/
/*	Brief : The delay routine
 *	\param : delay in ms
*/
void BMP180_delay_msek(u32 msek);
/* This function is an example for reading sensor data
 *	\param: None
 *	\return: communication result
 */
s32 bmp180_data_readout_template(void);
/*----------------------------------------------------------------------------
 struct bmp180_t parameters can be accessed by using bmp180
 *	bmp180_t having the following parameters
 *	Bus write function pointer: BMP180_WR_FUNC_PTR
 *	Bus read function pointer: BMP180_RD_FUNC_PTR
 *	Delay function pointer: delay_msec
 *	I2C address: dev_addr
 *	Chip id of the sensor: chip_id
 *	Calibration parameters
 ---------------------------------------------------------------------------*/
struct bmp180_t bmp180;
/*----------------------------------------------------------------------------*/
/* This function is an example for reading sensor data
 *	\param: None
 *	\return: communication result
 */
s32 bmp180_data_readout_template(void)
{
 /* result of communication results*/
	s32 com_rslt = E_BMP_COMM_RES;
	u16 v_uncomp_temp_u16 = BMP180_INIT_VALUE;
	u32 v_uncomp_press_u32 = BMP180_INIT_VALUE;
/*********************** START INITIALIZATION ************************/
    /**************Call the I2C init routine ***************/
	#ifdef BMP180_API
	I2C_routine();
	#endif
/*--------------------------------------------------------------------------
 *  This function used to assign the value/reference of
 *	the following parameters
 *	I2C address
 *	Bus Write
 *	Bus read
 *	Chip id
 *	Calibration values
-------------------------------------------------------------------------*/
	com_rslt = bmp180_init(&bmp180);

/************************* END INITIALIZATION *************************/
/*------------------------------------------------------------------*
************************* START CALIPRATION ********
*---------------------------------------------------------------------*/
	/*  This function used to read the calibration values of following
	 *	these values are used to calculate the true pressure and temperature
	 *	Parameter		MSB		LSB		bit
	 *		AC1			0xAA	0xAB	0 to 7
	 *		AC2			0xAC	0xAD	0 to 7
	 *		AC3			0xAE	0xAF	0 to 7
	 *		AC4			0xB0	0xB1	0 to 7
	 *		AC5			0xB2	0xB3	0 to 7
	 *		AC6			0xB4	0xB5	0 to 7
	 *		B1			0xB6	0xB7	0 to 7
	 *		B2			0xB8	0xB9	0 to 7
	 *		MB			0xBA	0xBB	0 to 7
	 *		MC			0xBC	0xBD	0 to 7
	 *		MD			0xBE	0xBF	0 to 7*/
	com_rslt += bmp180_get_calib_param();
/*------------------------------------------------------------------*
************************* END CALIPRATION ********
*---------------------------------------------------------------------*/
/************************* START READ UNCOMPENSATED TEMPERATURE AND PRESSURE********/


	/*	This API is used to read the
	*	uncompensated temperature(ut) value*/
	v_uncomp_temp_u16 = bmp180_get_uncomp_temperature();

	/*	This API is used to read the
	*	uncompensated pressure(ut) value*/
	v_uncomp_press_u32 = bmp180_get_uncomp_pressure();

/************************* END READ UNCOMPENSATED TEMPERATURE AND PRESSURE********/


/************************* START READ TRUE TEMPERATURE AND PRESSURE********/
/****************************************************************************
 *	This API is used to read the
 *	true temperature(t) value input
 *	parameter as uncompensated temperature(ut)
 *
 ***************************************************************************/
	com_rslt += bmp180_get_temperature(v_uncomp_temp_u16);

/****************************************************************************
 *	This API is used to read the
 *	true pressure(p) value
 *	input parameter as uncompensated pressure(up)
 *
 ***************************************************************************/
	com_rslt += bmp180_get_pressure(v_uncomp_press_u32);

/************************* END READ TRUE TEMPERATURE AND PRESSURE********/
return com_rslt;
}

#ifdef BMP180_API
/*--------------------------------------------------------------------------*
*	The following function is used to map the I2C bus read, write, delay and
*	device address with global structure bmp180_t
*-------------------------------------------------------------------------*/
s8 I2C_routine(void) {
/*--------------------------------------------------------------------------*
 *  By using bmp180 the following structure parameter can be accessed
 *	Bus write function pointer: BMP180_WR_FUNC_PTR
 *	Bus read function pointer: BMP180_RD_FUNC_PTR
 *	Delay function pointer: delay_msec
 *	I2C address: dev_addr
 *--------------------------------------------------------------------------*/
	bmp180.bus_write = BMP180_I2C_bus_write;
	bmp180.bus_read = BMP180_I2C_bus_read;
	bmp180.dev_addr = BMP180_I2C_ADDR;
	bmp180.delay_msec = BMP180_delay_msek;

	return BMP180_INIT_VALUE;
}

/************** I2C buffer length ******/

#define	I2C_BUFFER_LEN 8
#define I2C0 5
/*-------------------------------------------------------------------*
*	This is a sample code for read and write the data by using I2C
*	Configure the below code to your I2C driver
*	The device address is defined in the bmp180.c file
*-----------------------------------------------------------------------*/
/*	\Brief: The function is used as I2C bus write
 *	\Return : Status of the I2C write
 *	\param dev_addr : The device address of the sensor
 *	\param reg_addr : Address of the first register, will data is going to be written
 *	\param reg_data : It is a value hold in the array,
 *		will be used for write the value into the register
 *	\param cnt : The no of byte of data to be write
 */
s8 BMP180_I2C_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
	s32 iError = BMP180_INIT_VALUE;
	u8 array[I2C_BUFFER_LEN];
	u8 stringpos = BMP180_INIT_VALUE;
	array[BMP180_INIT_VALUE] = reg_addr;
	for (stringpos = BMP180_INIT_VALUE; stringpos < cnt; stringpos++) 
	{
		array[stringpos + C_BMP180_ONE_U8X] = *(reg_data + stringpos);
	}
	/*
	* Please take the below function as your reference for
	* write the data using I2C communication
	* "IERROR = I2C_WRITE_STRING(DEV_ADDR, ARRAY, CNT+C_BMP180_ONE_U8X)"
	* add your I2C write function here
	* iError is an return value of I2C read function
	* Please select your valid return value
	* In the driver SUCCESS defined as BMP180_INIT_VALUE
    * and FAILURE defined as -C_BMP180_ONE_U8X
	* Note :
	* This is a full duplex operation,
	* The first read data is discarded, for that extra write operation
	* have to be initiated. For that cnt+C_BMP180_ONE_U8X operation done in the I2C write string function
	* For more information please refer data sheet SPI communication:
	*/
	TM_I2C_WriteMulti(I2C1,dev_addr,reg_addr,reg_data,cnt);
	return (s8)iError;
}

 /*	\Brief: The function is used as I2C bus read
 *	\Return : Status of the I2C read
 *	\param dev_addr : The device address of the sensor
 *	\param reg_addr : Address of the first register, will data is going to be read
 *	\param reg_data : This data read from the sensor, which is hold in an array
 *	\param cnt : The no of byte of data to be read
 */
s8 BMP180_I2C_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
	s32 iError = BMP180_INIT_VALUE;
	u8 array[I2C_BUFFER_LEN] = {BMP180_INIT_VALUE};
	u8 stringpos = BMP180_INIT_VALUE;
	array[BMP180_INIT_VALUE] = reg_addr;
	/* Please take the below function as your reference
	 * for read the data using I2C communication
	 * add your I2C rad function here.
	 * "IERROR = I2C_WRITE_READ_STRING(DEV_ADDR, ARRAY, ARRAY, C_BMP180_ONE_U8X, CNT)"
	 * iError is an return value of SPI write function
	 * Please select your valid return value
	 * In the driver SUCCESS defined as BMP180_INIT_VALUE
     * and FAILURE defined as -C_BMP180_ONE_U8X
	 */
	TM_I2C_ReadMulti(I2C1,dev_addr,reg_addr,reg_data,cnt);
	
	for (stringpos = BMP180_INIT_VALUE; stringpos < cnt; stringpos++) {
		*(reg_data + stringpos) = array[stringpos];
	}
	return (s8)iError;
}
/*	Brief : The delay routine
 *	\param : delay in ms
*/
void BMP180_delay_msek(u32 msek)
{
	/*Here you can write your own delay routine*/
	osDelay(msek);
}

#endif
