// -----------------------------------------------------------------------------------------------
// BME280 library
//
// A BMP280 will also work without humidity functionality with this library.
//
// uint8_t begin(uint8_t csPin)
// void getCompensationValues()
// void readRawTempAndPressure(int32_t *pressure, int32_t *temp)
// int32_t compensateTemp(int32_t adc_T, int32_t *t_fine)
// int32_t compensatePressure(int32_t adc_P, int32_t t_fine)
// int32_t getTemperature(int32_t *t_fine)
// int32_t getPressure(int32_t t_fine)
// uint8_t read(byte reg)
// void write(byte reg, byte value)
// int16_t calcElevation(int32_t pressure, int32_t basePressure)
//
// -----------------------------------------------------------------------------------------------

#include "Arduino.h"
#include <math.h>
#include <limits.h>
#include <SPI.h>
#include "BME280.h"

// -----------------------------------------------------------------------------------------------
// Configure SPI -- do this before each access
// -----------------------------------------------------------------------------------------------
void configureSPI()
{
  SPI.setDataMode(SPI_MODE0);
#if F_CPU == 8000000L
  SPI.setClockDivider(SPI_CLOCK_DIV8);		// 1 MHz
#else // assume 16MHz
  SPI.setClockDivider(SPI_CLOCK_DIV16);		// 1 MHz
#endif
}

// -----------------------------------------------------------------------------------------------
// begin(csPin) - initialization
//
// returns 0 if failure
// returns the chip ID if success and loads the compensation coefficients
// into the structure
// -----------------------------------------------------------------------------------------------
uint8_t BME280::begin(uint8_t csPin)
{

  csPort = portOutputRegister(digitalPinToPort(csPin));
  csMask = digitalPinToBitMask(csPin);
  pinMode(csPin, OUTPUT);
  *csPort |= csMask;				// CS inactive
  delay(10);					// allow for some idle time
  SPI.begin();

  chipID = read(BME280_CHIP_ID_REG);
  if ((chipID != BME280_CHIP_ID) && (chipID != BMP280_CHIP_ID))
    return 0;
  else {
    getCompensationValues();  			// load compensation values into struct
    return chipID;
  }  
}

// -----------------------------------------------------------------------------------------------
// Load compensation values.
// This is called by begin().
// -----------------------------------------------------------------------------------------------
void BME280::getCompensationValues()
{
  uint8_t *p;

  p = (uint8_t *)&dig;
  *csPort &= ~csMask;
  SPI.transfer(BME280_CALIB_1_REG_START);
  for (uint8_t reg=BME280_CALIB_1_REG_START; reg<=BME280_CALIB_1_REG_END; reg++)
    *p++ = SPI.transfer(0);				// registers 0x88 - 0x9F (temp and pressure)
  *csPort |= csMask;
    
  // Humidity coefficients -- BME280 only
  if (chipID == BME280_CHIP_ID) {
    *csPort &= ~csMask;
    SPI.transfer(BME280_CALIB_2_REG);
    dig.H1 = SPI.transfer(0);				// register 0xA1 (humidity)
    *csPort |= csMask;
    
    *csPort &= ~csMask;
    SPI.transfer(BME280_CALIB_3_REG_START);
    dig.H2 = SPI.transfer(0);				// register 0xE1
    dig.H2 |= (uint16_t)SPI.transfer(0) << 8;		// register 0xE2
    dig.H3 = SPI.transfer(0);				// register 0xE3
    dig.H4 = (uint16_t)SPI.transfer(0) << 4;		// register 0xE4
    uint8_t regE5 = SPI.transfer(0);			// register 0xE5
    dig.H4 |= (regE5 & 0x0F);
    dig.H5 = (uint16_t)SPI.transfer(0) << 4;		// register 0xE6
    dig.H5 |= (regE5 >> 4);
    dig.H6 = SPI.transfer(0);				// register 0xE7
    *csPort |= csMask;
  }
}

// -----------------------------------------------------------------------------------------------
// Read the raw humidity, pressure and temperature values
// -----------------------------------------------------------------------------------------------
void BME280::readRawValues(int32_t *pressure, int32_t *temp, uint16_t *humidity) {
  *csPort &= ~csMask;
  SPI.transfer(BME280_PRESSURE_MSB_REG);
  *pressure = (int32_t)SPI.transfer(0) << 16;
  *pressure |= (int32_t)SPI.transfer(0) << 8;
  *pressure |= SPI.transfer(0);
  *pressure = *pressure >> 4;
  *temp = (int32_t)SPI.transfer(0) << 16;
  *temp |= (int32_t)SPI.transfer(0) << 8;
  *temp |= SPI.transfer(0);
  *temp = *temp >> 4;
  if (chipID == BME280_CHIP_ID) {
    *humidity = (int16_t)SPI.transfer(0) << 8;
    *humidity |= SPI.transfer(0);
  } else {
    *humidity = 0;
  }
  *csPort |= csMask;
}

// -----------------------------------------------------------------------------------------------
// Returns temperature hundredths of a degree Celsius. Output value of “5123” equals 51.23 DegC.
// The returned parameter t_fine is required by compensatePress().
//
// t_fine is returned as a parameter; it is needed for compensating the pressure.
// -----------------------------------------------------------------------------------------------
int32_t BME280::compensateTemp(int32_t adc_T, int32_t *t_fine)
{
  int32_t var1, var2, T;

  var1 = ((((adc_T>>3) - ((int32_t)dig.T1<<1))) * ((int32_t)dig.T2)) >> 11;
  var2 = (((((adc_T>>4) - ((int32_t)dig.T1)) * ((adc_T>>4) - ((int32_t)dig.T1))) >> 12) *
  	((int32_t)dig.T3)) >> 14;
  *t_fine = var1 + var2;
  T =(*t_fine*5+128)>>8;
  return T;
}

// -----------------------------------------------------------------------------------------------
// Returns pressure in Pa as an unsigned 32 bit integer -- 32 bit version.
//
// Input is the raw pressure and also t_fine, obtained from the temperature compensation.
// Output value of “96386” equals 96386 Pa = 963.86 hPa
// -----------------------------------------------------------------------------------------------
int32_t BME280::compensatePressure(int32_t adc_P, int32_t t_fine)
{
  int32_t var1, var2;
  uint32_t p;
 
  var1 = (((int32_t)t_fine)>>1) - (int32_t)64000;
  var2 = (((var1>>2) * (var1>>2)) >> 11 ) * ((int32_t)dig.P6);
  var2 = var2 + ((var1*((int32_t)dig.P5))<<1);
  var2 = (var2>>2)+(((int32_t)dig.P4)<<16);
  var1 = (((dig.P3 * (((var1>>2) * (var1>>2)) >> 13 )) >> 3) + ((((int32_t)dig.P2) * var1)>>1))>>18;
  var1 =((((32768+var1))*((int32_t)dig.P1))>>15);
  if (var1 == 0)
  {
    return 0; // avoid exception caused by division by zero
  }
  p = (((uint32_t)(((int32_t)1048576)-adc_P)-(var2>>12)))*3125;
  if (p < 0x80000000)
  {
    p = (p << 1) / ((uint32_t)var1);
  }
  else
  {
    p = (p / (uint32_t)var1) * 2;
  }
  var1 = (((int32_t)dig.P9) * ((int32_t)(((p>>3) * (p>>3))>>13)))>>12;
  var2 = (((int32_t)(p>>2)) * ((int32_t)dig.P8))>>13;
  p = (uint32_t)((int32_t)p + ((var1 + var2 + dig.P7) >> 4));
  return p;
}

// -----------------------------------------------------------------------------------------------
// Returns humidity as unsigned 32 bit integer in Q22.10 format (22 integer and 10 fractional bits).
// E.G.: Output value of "47445" represents 47445/1024 = 46.333% RH.
//
// Input is the 16-bit raw humidity and also t_fine, obtained from the temperature compensation.
// -----------------------------------------------------------------------------------------------
int32_t BME280::compensateHumidity(uint16_t rawH, int32_t t_fine)
{
  int32_t var1;
  
  int32_t adc_H = rawH;
  
  var1 = (t_fine - ((int32_t)76800));
  var1 = (((((adc_H << 14) - (((int32_t)dig.H4) << 20) - (((int32_t)dig.H5) * var1)) +
	((int32_t)16384)) >> 15) * (((((((var1 * ((int32_t)dig.H6)) >> 10) * (((var1 *
	((int32_t)dig.H3)) >> 11) + ((int32_t)32768))) >> 10) + ((int32_t)2097152)) *
	((int32_t)dig.H2) + 8192) >> 14));
  var1 = (var1 - (((((var1 >> 15) * (var1 >> 15)) >> 7) * ((int32_t)dig.H1)) >> 4));
  var1 = (var1 < 0 ? 0 : var1);
  var1 = (var1 > 419430400 ? 419430400 : var1);
  return (int32_t)(var1 >> 12);
}

// -----------------------------------------------------------------------------------------------
// Get the compensated temperature -- read the chip and return compensated value
//
// t_fine is returned as a parameter; it is needed for compensating the pressure.
// -----------------------------------------------------------------------------------------------
int32_t BME280::getTemperature(int32_t *t_fine) {
  int32_t rawT;

  *csPort &= ~csMask;
  SPI.transfer(BME280_TEMP_MSB_REG);
  rawT = (int32_t)SPI.transfer(0) << 16;
  rawT |= (int32_t)SPI.transfer(0) << 8;
  rawT |= SPI.transfer(0);
  rawT = rawT >> 4;
  *csPort |= csMask;

  return compensateTemp(rawT, t_fine);
}

// -----------------------------------------------------------------------------------------------
// Get the compensated pressure -- read the chip and return the compensated value
//
// The value t_fine is required; it is obtained from the temperature compensation.
// -----------------------------------------------------------------------------------------------
int32_t BME280::getPressure(int32_t t_fine) {
  int32_t rawP;

  *csPort &= ~csMask;
  SPI.transfer(BME280_PRESSURE_MSB_REG);
  rawP = (int32_t)SPI.transfer(0) << 16;
  rawP |= (int32_t)SPI.transfer(0) << 8;
  rawP |= SPI.transfer(0);
  rawP = rawP >> 4;
  *csPort |= csMask;

  return compensatePressure(rawP, t_fine);
}

// -----------------------------------------------------------------------------------------------
// Get the compensated humidity -- read the chip and return the compensated value
// Returns humidity as unsigned 32 bit integer in Q22.10 format (22 integer and 10 fractional bits).
// E.G.: Output value of "47445" represents 47445/1024 = 46.333% RH.

// The value t_fine is required; it is obtained from the temperature compensation.
// -----------------------------------------------------------------------------------------------
int32_t BME280::getHumidity(int32_t t_fine) {
  int16_t rawH;

  *csPort &= ~csMask;
  SPI.transfer(BME280_HUMIDITY_MSB_REG);
  rawH = (int16_t)SPI.transfer(0) << 8;
  rawH |= SPI.transfer(0);
  *csPort |= csMask;

  return compensateHumidity(rawH, t_fine);
}

// -----------------------------------------------------------------------------------------------
//  Read an 8 bit value
// -----------------------------------------------------------------------------------------------
uint8_t BME280::read(byte reg)
{
  uint8_t value;

  configureSPI();
  *csPort &= ~csMask;
  SPI.transfer(reg | 0x80);		// bit 7 = 1 for read
  value = SPI.transfer(0);
  *csPort |= csMask;
  return value;
}

// -----------------------------------------------------------------------------------------------
//  Write an 8 bit value
// -----------------------------------------------------------------------------------------------
void BME280::write(byte reg, byte value)
{
  configureSPI();
  *csPort &= ~csMask;
  SPI.transfer(reg & 0x7F);		// bit 7 = 0 for write
  SPI.transfer(value);
  *csPort |= csMask;
}

// -----------------------------------------------------------------------------------------------
// Calculate the altitude (in feet) from the specified atmospheric pressure (in Pascals)
// A standard atmosphere is assumed.
//
// pressure		pressure of current location in Pascals
// basePressure		pressure at sea level (or reference elevation) in Pascals
// calcElevation 	returns the altitude in feet
// -----------------------------------------------------------------------------------------------
int16_t BME280::calcElevation(int32_t pressure, int32_t basePressure)
{
  return((int)(145443.0*(1-pow(((float)pressure/(float)basePressure),(0.1903)))));
}


