// ===============================================================================================
// BME280 library
//
// Control registers:
//
// ----------------
// ctrl_meas (0xF4)
// ----------------
//    bits 7-5:  osrs_t (temperature oversampling)
//      000  skipped (output set to 0x80000)
//      001  x1
//      010  x2
//      011  x4
//      100  x8
//      101  x16
//      110  x16
//      111  x16
//    bits 4-2:  osrs_p (pressure oversampling)
//      000  skipped (output set to 0x80000)
//      001  x1
//      010  x2
//      011  x4
//      100  x8
//      101  x16
//      110  x16
//      111  x16      
//    bits 1-0:  mode
//      00   Sleep mode
//      01   Forced mode
//      10   Forced mode
//      11   Normal mode
//
// -------------
// config (0xF5)
// -------------
//   bits 7-5:  t_sb -- normal mode standby time
//      000  0.5ms
//      001  62.5
//      010  125
//      011  250
//      100  500
//      101  1000 (1s)
//      110  10		(2000ms for BMP280)
//      111  20		(4000ms for BMP280)
//   bits 4-2:  filter
//      000  off
//      001  2
//      010  4
//      011  8
//      1xx  16
//   bit 1:     -- unused --
//   bit 0:     spi3w en (enables 3-wire SPI when set to 1)
//
// ---------------
// ctrl_hum (0xF2)
// ---------------
//   bits 2-0:	osrs_h (humidity oversampling)
//	000 skipped (output set to 0x8000)
//	001 x1
//	010 x2
//	011 x4
//	100 x8
//	101 x16
//	110 x16
//	111 x16
// ===============================================================================================

#ifndef BME280_H
#define BME280_H

#include "Arduino.h"

// Register addresses
#define BME280_CALIB_1_REG_START	0x88	// first group of calibration coefficients
#define BME280_CALIB_1_REG_END		0x9F
#define BME280_CALIB_2_REG		0xA1	// single isolated calibration coefficient
#define BME280_CALIB_3_REG_START	0xE1	// third group of calibration coefficients
#define BME280_CALIB_3_REG_END		0xE7
#define BME280_CHIP_ID_REG		0xD0	// chip ID (read only)
#define BME280_CTRL_HUM_REG		0xF2
#define BME280_STATUS_REG		0xF3
#define BME280_CTRL_REG			0xF4	// control / measure
#define BME280_CONFIG_REG		0xF5
#define BME280_PRESSURE_MSB_REG		0xF7	// pressure MSB register
#define BME280_TEMP_MSB_REG		0xFA	// temperature MSB register
#define BME280_HUMIDITY_MSB_REG		0xFD	// humidity MSB register

// Chip ID
#define BME280_CHIP_ID			0x60
#define BMP280_CHIP_ID			0x58	// BMP280

// Control humidity (CTRL_HUM) register bit fields
// humidity oversample
#define BME280_HUM_OSRS_MASK		0x07
#define BME280_HUM_OSRS_OFF		0x00
#define BME280_HUM_OSRS_1X		0x01
#define BME280_HUM_OSRS_2X		0x02
#define BME280_HUM_OSRS_4X		0x03
#define BME280_HUM_OSRS_8X		0x04
#define BME280_HUM_OSRS_16X		0x05

// Control/Measure (CTRL_REG) register bit fields
// temperature oversample
#define BME280_TEMP_OSRS_MASK		0xE0
#define BME280_TEMP_OSRS_OFF		0x00
#define BME280_TEMP_OSRS_1X		0x20
#define BME280_TEMP_OSRS_2X		0x40
#define BME280_TEMP_OSRS_4X		0x60
#define BME280_TEMP_OSRS_8X		0x80
#define BME280_TEMP_OSRS_16X		0xE0
// pressure oversample
#define BME280_PRESS_OSRS_MASK		0x1C
#define BME280_PRESS_OSRS_OFF		0x00
#define BME280_PRESS_OSRS_1X		0x04
#define BME280_PRESS_OSRS_2X		0x08
#define BME280_PRESS_OSRS_4X		0x0C
#define BME280_PRESS_OSRS_8X		0x10
#define BME280_PRESS_OSRS_16X		0x1C
// mode
#define BME280_MODE_MASK		0x03
#define BME280_MODE_SLEEP		0x00
#define BME280_MODE_FORCED		0x01
#define BME280_MODE_NORMAL		0x03

// Config (CONFIG_REG) register bit fields
// standby time
#define BME280_STANDBY_MASK		0xE0
#define BME280_STANDBY_0_5_MS		0x00
#define BME280_STANDBY_62_5_MS		0x20
#define BME280_STANDBY_125_MS		0x40
#define BME280_STANDBY_250_MS		0x60
#define BME280_STANDBY_500_MS		0x80
#define BME280_STANDBY_1000_MS		0xA0
#define BME280_STANDBY_2000_MS		0xC0
#define BME280_STANDBY_4000_MS		0xE0
// filter setting
#define BME280_FILTER_MASK		0x1C
#define BME280_FILTER_OFF		0x00
#define BME280_FILTER_2			0x04
#define BME280_FILTER_4			0x08
#define BME280_FILTER_8			0x0C
#define BME280_FILTER_16		0x10
// 3 wire SPI enable
#define BME280_SPI3W_ENABLE		0x01

// Standard ISA sea level pressure
#define STD_SEA_LEVEL_PRESSURE		101325	// in Pascals


// Compensation coefficients

struct BME280_Compensation {
  uint16_t T1;			// 0x88 / 0x89 address
  int16_t  T2;			// 0x8A / 0x8B
  int16_t  T3;			// 0x8C / 0x8D
  uint16_t P1;			// 0x8E / 0x8F
  int16_t  P2;			// 0x90 / 0x91
  int16_t  P3;			// 0x92 / 0x93
  int16_t  P4;			// 0x94 / 0x95
  int16_t  P5;			// 0x96 / 0x97
  int16_t  P6;			// 0x98 / 0x99
  int16_t  P7;			// 0x9A / 0x9B
  int16_t  P8;			// 0x9C / 0x9D
  int16_t  P9;			// 0x9E / 0x9F
  uint8_t  H1;			// 0xA1
  int16_t  H2;			// 0xE1 / 0xE2
  uint8_t  H3;			// 0xE3
  int16_t  H4;			// 0xE4 / 0xE5[3:0] --> H4[11:0]=(E4|E5[3:0])
  int16_t  H5;			// 0xE5[7:4] / 0xE6 --> H5[11:0]=(E6|E5[7:4])
  int8_t   H6;			// 0xE7
};


class BME280
{
  public:
    uint8_t begin(uint8_t csPin);
    void getCompensationValues();
    void readRawValues(int32_t *pressure, int32_t *temp, uint16_t *humidity);
    int32_t compensateTemp(int32_t adc_T, int32_t *t_fine);
    int32_t compensatePressure(int32_t adc_P, int32_t t_fine);
    int32_t compensateHumidity(uint16_t rawH, int32_t t_fine);
    int32_t getTemperature(int32_t *t_fine);
    int32_t getPressure(int32_t t_fine);
    int32_t getHumidity(int32_t t_fine);
    uint8_t read(byte reg);
    void write(byte reg, byte value);
    int16_t calcElevation(int32_t pressure, int32_t basePressure);

  private:
    volatile uint8_t *csPort;			// digital i/o port for chip select
    uint8_t csMask;				// bit mask for chip select port
    uint8_t chipID;				// used to differentiate BME280 vs BMP280
    BME280_Compensation dig;			// compensation coefficients

};

#endif