/**	
 * |----------------------------------------------------------------------
 * | Copyright (C) Tilen Majerle, 2014
 * | 
 * | This program is free software: you can redistribute it and/or modify
 * | it under the terms of the GNU General Public License as published by
 * | the Free Software Foundation, either version 3 of the License, or
 * | any later version.
 * |  
 * | This program is distributed in the hope that it will be useful,
 * | but WITHOUT ANY WARRANTY; without even the implied warranty of
 * | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * | GNU General Public License for more details.
 * | 
 * | You should have received a copy of the GNU General Public License
 * | along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * |----------------------------------------------------------------------
 */
//#include "tm_stm32f4_dht11.h"
#include "DHT_Function.h"
#include "cmsis_os2.h"
#include "rtx_os.h"

#include "math.h"
#include "arm_math.h"

const DHT_msg_t DHT_Init = 
{
	.temperature=21.89,
	.humidity=55.0,
	.DewPoint=12.47,
	.GetComfortRatio=0.0,
	.HeatIndex=21.58,
	.Perception=0x01,
	.ComfortProfile=0x58,
	.ComfortStatus=0x0006,
	.Type=Celsius
};

const char DHT_Result_Message[8][20]=
{
	"A bit dry for some.",	//Trochu suchsie prostredie pre niektorych, ale velmi dobre prostredie ... ...
	"Very comfortable...",	//Velmi dobre prostredie ...
	"Comfortable, OK ...",	//Dobre prostredie ...
	"OK for most at edge",	//Dobre prostredie uz velmi nie, je to tak na hrane ...
	"Somewhat uncomform.",	//Pre niekoho uz vlhke prostredie ...
	"Very humid, quite.!",	//Velmi vlhke prostredie, opusti priestor ...
	"Extrem. uncomform.!",	//Extremne nevhodne prostredie, neodporuca sa dlhsi pobyt ...
	"Asthma, illnesses !"		//Absulutne extremne nevhodne prostredie, hrozi tu astma a porucha zraku ...
};

//void DHT_Compute_Propiertys(DHT_msg_t* DHT, float32_t temperature, float32_t humidity, float32_t pressure)
void DHT_Compute_Propiertys(DHT_msg_t* DHT)
{
 DHT->Type=Celsius;
 DHT->Altitude=DHT_Compute_Altitude(DHT->pressure);
 DHT->PressureAtSeaLevel=DHT_ComputeGetPressureAtSeaLevel(DHT->Altitude,DHT->pressure);
 DHT->HeatIndex=DHT_ComputeHeatIndex(DHT->temperature,DHT->humidity,DHT->Type);
 DHT->DewPoint=DHT_ComputeDewPoint(DHT->temperature,DHT->humidity,DHT->Type);
 DHT->Perception=DHT_ComputePerception(DHT->temperature,DHT->humidity,DHT->Type);
 DHT->GetComfortRatio=DHT_GetComfortRatio(&DHT->ComfortStatus,DHT->temperature,DHT->humidity,DHT->Type);
 DHT->ComfortProfile=DHT_GetComfortProfile(DHT->temperature,DHT->humidity,DHT->Type);
}

double_t DHT_convertCtoF(double_t c) 
{
  return (c * 1.8 + 32);
}

double_t DHT_convertFtoC(double_t f) 
{
  return ((f - 32) * 0.55555);
}

double_t DHT_Compute_Altitude(float32_t p)
{
	return ((float32_t)44330.0 * (float32_t)((float32_t)1.0 - (float32_t)pow((float32_t)p * ((float32_t) 0.00000986923266726), 1.0/5.255)));
}

double_t DHT_ComputeGetPressureAtSeaLevel(float32_t pressure, float32_t altitude) 
{
	return (pressure / ((float32_t)pow(1 - (float32_t)altitude / (float32_t)44330, 5.255)));
}

double_t DHT_ComputeHeatIndex(double_t temperature, double_t percentHumidity, int16_t isFahrenheit) 
{
// Using both Rothfusz and Steadman's equations
// http://www.wpc.ncep.noaa.gov/html/heatindex_equation.shtml
  double_t hi;

	if(!isFahrenheit) temperature = DHT_convertCtoF(temperature);

  hi = 0.5 * (temperature + 61.0 + ((temperature - 68.0) * 1.2) + (percentHumidity * 0.094));

  if(hi > 79) 
	{
    hi = -42.379 +
             2.04901523 * temperature +
            10.14333127 * percentHumidity +
            -0.22475541 * temperature*percentHumidity +
            -0.00683783 * pow(temperature, 2) +
            -0.05481717 * pow(percentHumidity, 2) +
             0.00122874 * pow(temperature, 2) * percentHumidity +
             0.00085282 * temperature*pow(percentHumidity, 2) +
            -0.00000199 * pow(temperature, 2) * pow(percentHumidity, 2);

      if((percentHumidity < 13) && (temperature >= 80.0) && (temperature <= 112.0)) hi -= ((13.0 - percentHumidity) * 0.25) * sqrt((17.0 - (double_t)fabs(temperature - 95.0)) * 0.05882);
			else
			if((percentHumidity > 85.0) && (temperature >= 80.0) && (temperature <= 87.0)) hi += ((percentHumidity - 85.0) * 0.1) * ((87.0 - temperature) * 0.2);
  }
	return(isFahrenheit ? hi : DHT_convertFtoC(hi));
}

float32_t DHT_ComputeDewPoint(float32_t temperature, float32_t percentHumidity, int16_t isFahrenheit) 
{
  // reference: http://wahiduddin.net/calc/density_algorithms.htm
  if(isFahrenheit) temperature = DHT_convertFtoC(temperature);
  double_t A0 = 373.15 / (273.15 + (double_t) temperature);
  double_t SUM = -7.90298 * (A0 - 1);
  SUM += 5.02808 * log10(A0);
  SUM += -1.3816e-7 * (pow(10, (11.344 * (1 - 1 / A0))) - 1) ;
  SUM += 8.1328e-3 * (pow(10, (-3.49149 * (A0 - 1))) - 1) ;
  SUM += log10(1013.246);
  double_t VP = pow(10, SUM - 3) * (double_t) percentHumidity;
  double_t Td = log(VP / 0.61078); // temp var
  Td = (241.88 * Td) / (17.558 - Td);
  return isFahrenheit ? DHT_convertCtoF(Td) : Td;
}

uint8_t DHT_ComputePerception(float32_t temperature, float32_t huumidity, int16_t isFahrenheit) 
{
	float32_t dewPoint;
  // Computing human perception from dew point
  // reference: https://en.wikipedia.org/wiki/Dew_point ==> Relationship to human comfort
  // reference: Horstmeyer, Steve (2006-08-15). "Relative Humidity....Relative to What? The Dew Point Temperature...a better approach". Steve Horstmeyer, Meteorologist, WKRC TV, Cincinnati, Ohio, USA. Retrieved 2009-08-20.
  // Using table
  // Return value Dew point    Human perception[6]
  //    7         Over 26 C   Severely high, even deadly for Asthma related illnesses
  //    6         2426 C     Extremely uncomfortable, oppressive
  //    5         2124 C     Very humid, quite uncomfortable
  //    4         1821 C     Somewhat uncomfortable for most people at upper edge
  //    3         1618 C     OK for most, but all perceive the humidity at upper edge
  //    2         1316 C     Comfortable
  //    1         1012 C     Very comfortable
  //    0         Under 10 C  A bit dry for some

  if (isFahrenheit) 
	 {
    temperature = DHT_convertFtoC(temperature);
   }
   dewPoint = DHT_ComputeDewPoint(temperature, huumidity, isFahrenheit);

  if (dewPoint < 10.0f) 
	{
    return DHT_Perception_Dry;
  } else if (dewPoint < 13.0f) 
	{
    return DHT_Perception_VeryComfy;
  } else if (dewPoint < 16.0f) 
	{
    return DHT_Perception_Comfy;
  } else if (dewPoint < 18.0f) 
	{
    return DHT_Perception_Ok;
  } else if (dewPoint < 21.0f) 
	{
    return DHT_Perception_UnComfy;
  } else if (dewPoint < 24.0f) 
	{
    return DHT_Perception_QuiteUnComfy;
  } else if (dewPoint < 26.0f) 
	{
    return DHT_Perception_VeryUnComfy;
  }
  // else dew >= 26.0
  return DHT_Perception_SevereUncomfy;
}

float32_t DHT_distanceTooHot(float32_t temp, float32_t humidity)
{
	float32_t m_tooHot_m, m_tooHot_b;
	return temp - (humidity * m_tooHot_m + m_tooHot_b);
}

float32_t DHT_distanceTooHumid(float32_t temp, float32_t humidity)
{
	float32_t m_tooHumid_m, m_tooHumid_b;
	return temp - (humidity * m_tooHumid_m + m_tooHumid_b);
}

float32_t DHT_distanceTooCold(float32_t temp, float32_t humidity)
{
	float32_t m_tooCold_m, m_tooHCold_b;
	return (humidity * m_tooCold_m + m_tooHCold_b) - temp;
}

float32_t DHT_distanceTooDry(float32_t temp, float32_t humidity)
{
	float32_t m_tooDry_m, m_tooDry_b;
	return (humidity * m_tooDry_m + m_tooDry_b) - temp;
}

enum {isTooHot=1, isTooHumid=2, isTooCold=4, isTooDry=8};

const ComfortProfile DHT_m_comfort =
{	//Set default comfort profile.
  //In computing these constants the following reference was used
  //http://epb.apogee.net/res/refcomf.asp
  //It was simplified as 4 straight lines and added very little skew on
  //the vertical lines (+0.1 on x for C,D)
  //The for points used are(from top left, clock wise)
  //A(30%, 30*C) B(70%, 26.2*C) C(70.1%, 20.55*C) D(30.1%, 22.22*C)
  //On the X axis we have the rel humidity in % and on the Y axis the temperature in *C
  //Too hot line AB
	.m_tooHot_m = -0.095,
  .m_tooHot_b = 32.85,
  //Too humid line BC
  .m_tooHumid_m =  -56.5,
  .m_tooHumid_b = 3981.2,
  //Too cold line DC
  .m_tooCold_m = -0.04175,
  .m_tooHCold_b = 23.476675,
  //Too dry line AD
  .m_tooDry_m = -77.8,
  .m_tooDry_b = 2364
};

uint8_t DHT_GetComfortProfile(float32_t temperature, float32_t humidity, int16_t isFahrenheit)
{
	uint8_t Profile = 0x00;
  if(isFahrenheit) temperature = DHT_convertFtoC(temperature);
  if(temperature > (humidity * DHT_m_comfort.m_tooHot_m + DHT_m_comfort.m_tooHot_b)) Profile|=isTooHot;
  if(temperature > (humidity * DHT_m_comfort.m_tooHumid_m + DHT_m_comfort.m_tooHumid_b)) Profile|=isTooHumid;
  if(temperature < (humidity * DHT_m_comfort.m_tooCold_m + DHT_m_comfort.m_tooHCold_b)) Profile|=isTooCold;
  if(temperature < (humidity * DHT_m_comfort.m_tooDry_m + DHT_m_comfort.m_tooDry_b)) Profile|=isTooDry;
	return(Profile);
}

float32_t DHT_GetComfortRatio(int16_t *destComfortStatus, float32_t temperature, float32_t percentHumidity, int16_t isFahrenheit) 
{
	float32_t ratio = 100; 				//100%
	float32_t distance = 0;
	float32_t kTempFactor = 3;		//take into account the slope of the lines
	float32_t kHumidFactor = 0.1; //take into account the slope of the lines

	uint8_t tempComfort = 0;

  if(isFahrenheit) temperature = DHT_convertFtoC(temperature);

	*destComfortStatus = DHT_Comfort_OK;

	distance = DHT_distanceTooHot(temperature, percentHumidity);
	if(distance > 0)
	{
		//update the comfort descriptor
		tempComfort += (uint8_t)DHT_Comfort_TooHot;
		//decrease the comfot ratio taking the distance into account
		ratio -= distance * kTempFactor;
	}

	distance = DHT_distanceTooHumid(temperature, percentHumidity);
	if(distance > 0)
	{
		//update the comfort descriptor
		tempComfort += (uint8_t)DHT_Comfort_TooHumid;
		//decrease the comfot ratio taking the distance into account
		ratio -= distance * kHumidFactor;
	}

	distance = DHT_distanceTooCold(temperature, percentHumidity);
	if(distance > 0)
	{
		//update the comfort descriptor
		tempComfort += (uint8_t)DHT_Comfort_TooCold;
		//decrease the comfot ratio taking the distance into account
		ratio -= distance * kTempFactor;
	}

	distance = DHT_distanceTooDry(temperature, percentHumidity);
	if(distance > 0)
	{
		//update the comfort descriptor
		tempComfort += (uint8_t)DHT_Comfort_TooDry;
		//decrease the comfot ratio taking the distance into account
		ratio -= distance * kHumidFactor;
	}

	*destComfortStatus = tempComfort;

	if(ratio < 0) ratio = 0;

	return(ratio);
}

