/**
 * @author  Tilen Majerle
 * @email   tilen@majerle.eu
 * @website http://stm32f4-discovery.com
 * @link    http://stm32f4-discovery.com/2014/08/library-25-am2301-dht21-sensor-stm32f4xx/
 * @version v1.3
 * @ide     Keil uVision
 * @license GNU GPL v3
 * @brief   Basic library for DHT21 (DHT21) temperature and humidity sensor
 *	
@verbatim
   ----------------------------------------------------------------------
    Copyright (C) Tilen Majerle, 2015
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.
     
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
   ----------------------------------------------------------------------
@endverbatim
 */
#ifndef TM_DHT21_H
#define TM_DHT21_H 130

/* C++ detection */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * @addtogroup TM_STM32F4xx_Libraries
 * @{
 */

/**
 * @defgroup TM_DHT21
 * @brief    Basic library for DHT21 (DHT21) temperature and humidity sensor - http://stm32f4-discovery.com/2014/08/library-25-am2301-dht21-sensor-stm32f4xx/
 * @{
 *
 * This library does not use any timers and interrupts to increase speed, only basic pooling mode.
 * 
 * \par Default pinout
 *
@verbatim
DHT21		STM32F4xx		Description

VCC			3V3-5V			Supply voltage from 3V3 to 5V
GND			GND				Ground
DATA		PD1				Data line
@endverbatim
 *
 * By default, PD1 pin is used for data. If you want to change it, use lines below in defines.h to and edit them:
 *
@verbatim
//Select custom pin for DHT21 sensor
#define DHT21_PORT				GPIOD
#define DHT21_PIN				GPIO_PIN_1
@endverbatim
 *
 * \par Data output
 *
 * Temperature and humidity are returned from sensor in x10 multiplier, so like 55.5% humidity,
 * sensor will return 555 and 27.3C, sensor will return 273.
 *
 * @note This values are also returned from my library, you have to manually convert them (divide by 10)
 *
 * \par Changelog
 *
@verbatim
 Version 1.3
  - March 12, 2015
  - Added support for my new GPIO library

 Version 1.2
  - December 01, 2014
  - Added comments

 Version 1.1
  - November 28, 2014
  - Designed for new Delay system
 
 Version 1.0
  - First release
@endverbatim
 *
 * \par Dependencies
 *
@verbatim
 - STM32F4xx
 - defines.h
 - TM DELAY
 - TM GPIO
 
@endverbatim
 */
/**
 * Dependencies
 * - STM32F4xx
 * - defines.h
 * - TM DELAY
 * - TM GPIO
 */
/**
 * Includes
 */
#include "stm32f4xx.h"
#include "defines.h"
#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_gpio.h"

/**
 * @defgroup TM_DHT21_Macros
 * @brief    Library defines
 * @{
 */

/**
 * @brief  Default data pin, overwrite it in defines.h file 
 */
#ifndef DHT21_PIN
#define DHT21_PORT				GPIOD
#define DHT21_PIN				GPIO_PIN_11
#endif

/* Pin settings */
#define DHT21_PIN_LOW			TM_GPIO_SetPinLow(DHT21_PORT, DHT21_PIN)
#define DHT21_PIN_HIGH			TM_GPIO_SetPinHigh(DHT21_PORT, DHT21_PIN)
#define DHT21_PIN_IN				TM_GPIO_SetPinAsInput(DHT21_PORT, DHT21_PIN)
#define DHT21_PIN_OUT			TM_GPIO_SetPinAsOutput(DHT21_PORT, DHT21_PIN)
#define DHT21_PIN_READ			TM_GPIO_GetInputPinValue(DHT21_PORT, DHT21_PIN)

/**
 * @}
 */
 
/**
 * @defgroup TM_DHT21_Typedefs
 * @brief    Library Typedefs
 * @{
 */

/**
 * Enumerations
 *
 * There are several different possible results.
 * If TM_DHT21_OK is returned from read function then you have valid data.
 */
typedef enum {
	TM_DHT21_OK,						/*!< Data OK */
	TM_DHT21_ERROR,					/*!< An error occurred */
	TM_DHT21_CONNECTION_ERROR,			/*!< Device is not connected */
	TM_DHT21_WAITHIGH_ERROR,           /*!< Wait high pulse timeout */
	TM_DHT21_WAITLOW_ERROR,            /*!< Wait low pulse timeout */
	TM_DHT21_WAITHIGH_LOOP_ERROR,      /*!< Loop error for high pulse */
 	TM_DHT21_WAITLOW_LOOP_ERROR,       /*!< Loop error for low pulse */
	TM_DHT21_PARITY_ERROR				/*!< Data read fail */
} TM_DHT21_t;

/**
 * @brief  DHT21 main data structure		
 */
typedef struct 
{
	int16_t Temp; /*!< Temperature in tenths of degrees.
	                   If real temperature is 27.3C, this variable's value is 273 */
	uint16_t Hum; /*!< Humidity in tenths of percent.
	                   If real humidity is 55.5%, this variable's value is 555 */
} 
TM_DHT21_Data_t;

/**
 * @}
 */

/**
 * @defgroup TM_DHT21_Functions
 * @brief    Library Functions
 * @{
 */

/**
 * @brief  Initializes DHT21 sensor
 * @param  None
 * @retval TM_DHT21_OK
 */
TM_DHT21_t TM_DHT21_Init(void);

/**
 * @brief  Reads data from sensor
 * @param  *data: Pointer to @ref TM_DHT21_Data_t data structure to store data into
 * @retval Data valid:
 *            - TM_DHT21_OK: Data valid
 *            - Else: Data not valid
 */
TM_DHT21_t TM_DHT21_Read(TM_DHT21_Data_t* data);

/**
 * @}
 */
 
/**
 * @}
 */
 
/**
 * @}
 */

/* C++ detection */
#ifdef __cplusplus
}
#endif


#endif

