#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pidreg.h"
#include "arm_math.h"

/*
Purpose : This file contains the main body of the PID controller.
For design details, please read the Word document
"PID Controller Calculus".
In the GUI, the following parameters can be changed:
kc: The controller gain
Ti: Time-constant for the Integral Gain
Td: Time-constant for the Derivative Gain
Ts: The sample period [seconds]
*/


//--------------------
// Function Prototypes
//--------------------

void init_pid_k0(pid_params *p);
void reg_pid_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_pid_v0(pid_params *p);
void reg_pid_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_ale_k0(pid_params *p);
void reg_ale_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_ale_v0(pid_params *p);
void reg_ale_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_tak_k0(pid_params *p);
void reg_tak_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_tak_v0(pid_params *p);
void reg_tak_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_pid_k1(pid_params *p);
void reg_pid_k1(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_pid_v1(pid_params *p);
void reg_pid_v1(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_ale_k1(pid_params *p);
void reg_ale_k1(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_ale_v1(pid_params *p);
void reg_ale_v1(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_tak_k1(pid_params *p);
void reg_tak_k1(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_tak_v1(pid_params *p);
void reg_tak_v1(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_dsp_k0(pid_params *p);
void reg_dsp_k0(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_dsp_v0(pid_params *p);
void reg_dsp_v0(double_t xk, double_t *yk, pid_params *p, var_params *v);

void init_dsp_k1(pid_params *p);
void reg_dsp_k1(double_t xk, double_t *yk, pid_params *p, var_params *v);
void init_dsp_v1(pid_params *p);
void reg_dsp_v1(double_t xk, double_t *yk, pid_params *p, var_params *v);

/*------------------------------------------------------------------
Purpose : This function initialises the PID controller, based on
the new Type A PID controller.
Variables: p: pointer to struct containing all PID parameters
							Ts 				2.Td
k0 = kc.(1 + ---- + ------------)
							2.Ti Ts + 2.k_lpf
							
					Ts.Ts/Ti  4.k_lpf - 4.Td
k1 = kc.( ------------------------- )
							Ts + 2.k_lpf

					2.k_lpf  Ts + Ts.Ts/(2.Ti)  k_lpf.Ts/Ti + 2.Td
k2 = kc.( ------------------------------------------------ )
													Ts + 2.k_lpf
Returns : No values are returned
------------------------------------------------------------------*/

void init_pid_k0(pid_params *p)
{
	const double_t alfa = p->ts + 2.0 * p->k_lpf; // help variable
	//p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
	if (p->ti > 0.001)
	 {
		p->k0 = p->kc * (+1.0 + (p->ts / (2.0 * p->ti)) + (2.0 * p->td / alfa));
		p->k1 = p->kc * (p->ts * p->ts / p->ti - 4.0 * p->k_lpf - 4.0 * p->td);
		p->k1 /= alfa;
		p->k2 = 2.0 * p->k_lpf - p->ts + p->ts * p->ts / (2.0 * p->ti);
		p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;												//Bola tu chyba !!! p->k2 += 2.0 * p->ti - p->k_lpf * p->ts / p->ti; co je teraz spravne ??? p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;
		p->k2 *= p->kc / alfa;
	 }
	p->lpf1 = 4.0 * p->k_lpf / alfa;
	p->lpf2 = (p->ts - 2.0 * p->k_lpf) / alfa;
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID controller.
It is an update of pid_reg1(), derived with Bilinear
Transformation. It is a Type A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_pid_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																						// e[k]
	ek = p->Soll - xk; 																		// calculate e[k]
	if (p->pid_model)
		{
		*yk = p->lpf1 * v->yk_1 + p->lpf2 * v->yk_2;			// y[k] = p1*y[k-1] + p2*y[k-2]
		*yk += p->k0 * ek; 																// ... + k0 * e[k]
		*yk += p->k1 * v->ek_1;														// ... + k1 * e[k-1]
		*yk += p->k2 * v->ek_2;														// ... + k2 * e[k-2]
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	v->yk_1 = *yk;																			// y[k-1] = yk
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
	
} 

void init_pid_v0(pid_params *p)
{
	const double_t alfa = p->ts + 2.0 * p->k_lpf; // help variable
	//p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
	if (p->ti > 0.001)
	 {
		p->k0 = p->kc * (+1.0 + (p->ts / (2.0 * p->ti)) + (2.0 * p->td / alfa));
		p->k1 = p->kc * (p->ts * p->ts / p->ti - 4.0 * p->k_lpf - 4.0 * p->td);
		p->k1 /= alfa;
		p->k2 = 2.0 * p->k_lpf - p->ts + p->ts * p->ts / (2.0 * p->ti);
		p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;												//Bola tu chyba !!! p->k2 += 2.0 * p->ti - p->k_lpf * p->ts / p->ti;
		p->k2 *= p->kc / alfa;
	 }
	p->lpf1 = 4.0 * p->k_lpf / alfa;
	p->lpf2 = (p->ts - 2.0 * p->k_lpf) / alfa;
} 

void reg_pid_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																					// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	if (p->pid_model)
		{
		*yk = p->lpf1 * v->yk_1 + p->lpf2 * v->yk_2;			// y[k] = p1*y[k-1] + p2*y[k-2]
		*yk += p->k0 * ek; 																// ... + k0 * e[k]
		*yk += p->k1 * v->ek_1;														// ... + k1 * e[k-1]
		*yk += p->k2 * v->ek_2;														// ... + k2 * e[k-2]
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	v->yk_1 = *yk;																			// y[k-1] = yk
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
	
}

void init_ale_k0(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]
Returns : No values are returned
------------------------------------------------------------------*/
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_ale_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the type Allen Bradley Type A PID              
controller. All terms are dependent on the error signal e[k]. 
The D term is also low-pass filtered. 
This function should be called once every TS seconds. 
Variables:        
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature         
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
  ------------------------------------------------------------------*/ 
{
   double_t ek;  			// e[k]    
	 double_t lpf; 			//LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
   //-------------------------------------- 
   // Calculate Lowpass Filter for D-term    
	 //--------------------------------------    
	 lpf = p->lpf1 * v->lpf_1 + p->lpf2 * (ek + v->ek_1); 
    if (p->pid_model) 
   { 	//----------------------------------------------------------- 
      // Calculate PID controller: 
      // y[k] = y[k-1] + Kc*(e[k] - e[k-1] + 
      // Ts*e[k]/Ti + 
      // Td/Ts*(lpf[k] - 2*lpf[k-1]+lpf[k-2]))       
			//-----------------------------------------------------------      
			p->pp = p->kc * (ek - v->ek_1);									// y[k] = y[k-1] + Kc*(e[k] - e[k-1])       
			p->pi = p->k0 * ek;           									//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (lpf - 2.0 * v->lpf_1 + v->lpf_2); 
      *yk += p->pp + p->pi + p->pd; 
   }
	 else *yk = +0.0; 
   v->ek_1 = ek;    																	// e[k-1] = e[k]    
	 v->lpf_2 = v->lpf_1; 															// update stores for LPF    
	 v->lpf_1 = lpf;
	 v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	 v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM) 
    { 
     *yk = GMA_HLIM; 
    }
		 else 
		 if (*yk < GMA_LLIM) 
     { 
      *yk = GMA_LLIM; 
     } // else 
	
}

void init_ale_v0(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_ale_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the type Allen Bradley Type A PID              
controller. All terms are dependent on the error signal e[k]. 
The D term is also low-pass filtered. 
This function should be called once every TS seconds. 
Variables:        
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature         
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
  ------------------------------------------------------------------*/ 
{
   double_t ek;  			// e[k]    
	 double_t lpf; 			//LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
   //-------------------------------------- 
   // Calculate Lowpass Filter for D-term    
	 //--------------------------------------    
	 lpf = p->lpf1 * v->lpf_1 + p->lpf2 * (ek + v->ek_1); 
    if (p->pid_model) 
   {  //----------------------------------------------------------- 
      // Calculate PID controller: 
      // y[k] = y[k-1] + Kc*(e[k] - e[k-1] + 
      // Ts*e[k]/Ti + 
      // Td/Ts*(lpf[k] - 2*lpf[k-1]+lpf[k-2]))       
			//-----------------------------------------------------------      
			p->pp = p->kc * (ek - v->ek_1);									// y[k] = y[k-1] + Kc*(e[k] - e[k-1])       
			p->pi = p->k0 * ek;           									//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (lpf - 2.0 * v->lpf_1 + v->lpf_2); 
      *yk += p->pp + p->pi + p->pd; 
   }
	 else *yk = +0.0; 
   v->ek_1 = ek;    																	// e[k-1] = e[k]    
	 v->lpf_2 = v->lpf_1; 															// update stores for LPF    
	 v->lpf_1 = lpf; 
	 v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	 v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM) 
    { 
     *yk = GMA_HLIM; 
    }
		 else 
		 if (*yk < GMA_LLIM) 
     { 
      *yk = GMA_LLIM; 
     } // else 
	
}

void init_tak_k0(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_tak_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the Takahashi PID controller,              
which is a type C controller: the P and D term are no              
longer dependent on the set-point, only on PV (which is Thlt). 
The D term is NOT low-pass filtered. 
This function should be called once every TS seconds. 
Variables:
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
------------------------------------------------------------------*/ 
{ double_t ek;  // e[k]    
	//double_t lpf; //LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
    if (p->pid_model) 
   { 
      //----------------------------------------------------------- 
      //Calculate PID controller: 
      //y[k] = y[k-1] + Kc*(PV[k-1] - PV[k] + 
      //Ts*e[k]/Ti + 
      //Td/Ts*(2*PV[k-1] - PV[k] - PV[k-2])) 
      //-----------------------------------------------------------       
			p->pp = p->kc * (v->xk_1 - xk);										// y[k] = y[k-1] + Kc*(PV[k-1] - PV[k])       
			p->pi = p->k0 * ek;           										//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (2.0 * v->xk_1 - xk - v->xk_2); 
      *yk += p->pp + p->pi + p->pd; 
   }    else 
					{
					 *yk = p->pp = p->pi = p->pd = 0.0; 
					} 
    v->xk_2 = v->xk_1;		// PV[k-2] = PV[k-1]    
		v->xk_1 = xk;    			// PV[k-1] = PV[k] 
		v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
		v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM)    
		{ 
     *yk = GMA_HLIM;    
		}    
			else 
			if (*yk < GMA_LLIM)    
			 { 
				*yk = GMA_LLIM; 
			 } // else 
	
}

void init_tak_v0(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 //p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_tak_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the Takahashi PID controller,              
which is a type C controller: the P and D term are no              
longer dependent on the set-point, only on PV (which is Thlt). 
The D term is NOT low-pass filtered. 
This function should be called once every TS seconds. 
Variables:
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
------------------------------------------------------------------*/ 
{ double_t ek;  // e[k]    
	//double_t lpf; //LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
    if (p->pid_model) 
   { 
      //----------------------------------------------------------- 
      //Calculate PID controller: 
      //y[k] = y[k-1] + Kc*(PV[k-1] - PV[k] + 
      //Ts*e[k]/Ti + 
      //Td/Ts*(2*PV[k-1] - PV[k] - PV[k-2])) 
      //-----------------------------------------------------------       
			p->pp = p->kc * (v->xk_1 - xk);										// y[k] = y[k-1] + Kc*(PV[k-1] - PV[k])       
			p->pi = p->k0 * ek;           										//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (2.0 * v->xk_1 - xk - v->xk_2); 
      *yk += p->pp + p->pi + p->pd; 
   }    else 
					{
					 *yk = p->pp = p->pi = p->pd = 0.0; 
					} 
    v->xk_2 = v->xk_1;		// PV[k-2] = PV[k-1]    
		v->xk_1 = xk;    			// PV[k-1] = PV[k] 
		v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
		v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM)    
		{ 
     *yk = GMA_HLIM;    
		}    
			else 
			if (*yk < GMA_LLIM)    
			 { 
				*yk = GMA_LLIM; 
			 } // else 
	
}

//****************************************************************************************************************************************************************************************

/*------------------------------------------------------------------
Purpose : This function initialises the PID controller, based on
the new Type A PID controller.
Variables: p: pointer to struct containing all PID parameters
							Ts 				2.Td
k0 = kc.(1 + ---- + ------------)
							2.Ti Ts + 2.k_lpf
							
					Ts.Ts/Ti  4.k_lpf - 4.Td
k1 = kc.( ------------------------- )
							Ts + 2.k_lpf

					2.k_lpf  Ts + Ts.Ts/(2.Ti)  k_lpf.Ts/Ti + 2.Td
k2 = kc.( ------------------------------------------------ )
													Ts + 2.k_lpf
Returns : No values are returned
------------------------------------------------------------------*/

void init_pid_k1(pid_params *p)
{
	const double_t alfa = p->ts + 2.0 * p->k_lpf; // help variable
	//p->os_wait_ts_ticks=(unsigned char)(p->ts*1000);
	if (p->ti > 0.001)
	 {
		p->k0 = p->kc * (+1.0 + (p->ts / (2.0 * p->ti)) + (2.0 * p->td / alfa));
		p->k1 = p->kc * (p->ts * p->ts / p->ti - 4.0 * p->k_lpf - 4.0 * p->td);
		p->k1 /= alfa;
		p->k2 = 2.0 * p->k_lpf - p->ts + p->ts * p->ts / (2.0 * p->ti);
		p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;												//Bola tu chyba !!! p->k2 += 2.0 * p->ti - p->k_lpf * p->ts / p->ti; co je teraz spravne ??? p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;
		p->k2 *= p->kc / alfa;
	 }
	p->lpf1 = 4.0 * p->k_lpf / alfa;
	p->lpf2 = (p->ts - 2.0 * p->k_lpf) / alfa;
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID controller.
It is an update of pid_reg1(), derived with Bilinear
Transformation. It is a Type A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_pid_k1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																					// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	if (p->pid_model)
		{
		*yk = p->lpf1 * v->yk_1 + p->lpf2 * v->yk_2;			// y[k] = p1*y[k-1] + p2*y[k-2]
		*yk += p->k0 * ek; 																// ... + k0 * e[k]
		*yk += p->k1 * v->ek_1;														// ... + k1 * e[k-1]
		*yk += p->k2 * v->ek_2;														// ... + k2 * e[k-2]
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	v->yk_1 = *yk;																			// y[k-1] = yk
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
	
} 

void init_pid_v1(pid_params *p)
{
	const double_t alfa = p->ts + 2.0 * p->k_lpf; // help variable
	if (p->ti > 0.001)
	 {
		p->k0 = p->kc * (+1.0 + (p->ts / (2.0 * p->ti)) + (2.0 * p->td / alfa));
		p->k1 = p->kc * (p->ts * p->ts / p->ti - 4.0 * p->k_lpf - 4.0 * p->td);
		p->k1 /= alfa;
		p->k2 = 2.0 * p->k_lpf - p->ts + p->ts * p->ts / (2.0 * p->ti);
		p->k2 += 2.0 * p->td - p->k_lpf * p->ts / p->ti;												//Bola tu chyba !!! p->k2 += 2.0 * p->ti - p->k_lpf * p->ts / p->ti;
		p->k2 *= p->kc / alfa;
	 }
	p->lpf1 = 4.0 * p->k_lpf / alfa;
	p->lpf2 = (p->ts - 2.0 * p->k_lpf) / alfa;
} 

void reg_pid_v1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	double_t ek;																					// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	if (p->pid_model)
		{
		*yk = p->lpf1 * v->yk_1 + p->lpf2 * v->yk_2;			// y[k] = p1*y[k-1] + p2*y[k-2]
		*yk += p->k0 * ek; 																// ... + k0 * e[k]
		*yk += p->k1 * v->ek_1;														// ... + k1 * e[k-1]
		*yk += p->k2 * v->ek_2;														// ... + k2 * e[k-2]
		}
	else *yk = +0.0;
	v->ek_2 = v->ek_1;																	// e[k-2] = e[k-1]
	v->ek_1 = ek; 																			// e[k-1] = e[k]
	v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	v->yk_1 = *yk;																			// y[k-1] = yk
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
	
}

void init_ale_k1(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]
Returns : No values are returned
------------------------------------------------------------------*/
{
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_ale_k1(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the type Allen Bradley Type A PID              
controller. All terms are dependent on the error signal e[k]. 
The D term is also low-pass filtered. 
This function should be called once every TS seconds. 
Variables:        
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature         
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
  ------------------------------------------------------------------*/ 
{
   double_t ek;  			// e[k]    
	 double_t lpf; 			//LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
   //-------------------------------------- 
   // Calculate Lowpass Filter for D-term    
	 //--------------------------------------    
	 lpf = p->lpf1 * v->lpf_1 + p->lpf2 * (ek + v->ek_1); 
    if (p->pid_model) 
   { 	//----------------------------------------------------------- 
      // Calculate PID controller: 
      // y[k] = y[k-1] + Kc*(e[k] - e[k-1] + 
      // Ts*e[k]/Ti + 
      // Td/Ts*(lpf[k] - 2*lpf[k-1]+lpf[k-2]))       
			//-----------------------------------------------------------      
			p->pp = p->kc * (ek - v->ek_1);									// y[k] = y[k-1] + Kc*(e[k] - e[k-1])       
			p->pi = p->k0 * ek;           									//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (lpf - 2.0 * v->lpf_1 + v->lpf_2); 
      *yk += p->pp + p->pi + p->pd; 
   }
	 else *yk = +0.0; 
   v->ek_1 = ek;    																	// e[k-1] = e[k]    
	 v->lpf_2 = v->lpf_1; 															// update stores for LPF    
	 v->lpf_1 = lpf; 
	 v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	 v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM) 
    { 
     *yk = GMA_HLIM; 
    }
		 else 
		 if (*yk < GMA_LLIM) 
     { 
      *yk = GMA_LLIM; 
     } // else 
	
}

void init_ale_v1(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_ale_v1(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the type Allen Bradley Type A PID              
controller. All terms are dependent on the error signal e[k]. 
The D term is also low-pass filtered. 
This function should be called once every TS seconds. 
Variables:        
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature         
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
  ------------------------------------------------------------------*/ 
{
   double_t ek;  			// e[k]    
	 double_t lpf; 			//LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
   //-------------------------------------- 
   // Calculate Lowpass Filter for D-term    
	 //--------------------------------------    
	 lpf = p->lpf1 * v->lpf_1 + p->lpf2 * (ek + v->ek_1); 
    if (p->pid_model) 
   {  //----------------------------------------------------------- 
      // Calculate PID controller: 
      // y[k] = y[k-1] + Kc*(e[k] - e[k-1] + 
      // Ts*e[k]/Ti + 
      // Td/Ts*(lpf[k] - 2*lpf[k-1]+lpf[k-2]))       
			//-----------------------------------------------------------      
			p->pp = p->kc * (ek - v->ek_1);									// y[k] = y[k-1] + Kc*(e[k] - e[k-1])       
			p->pi = p->k0 * ek;           									//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (lpf - 2.0 * v->lpf_1 + v->lpf_2); 
      *yk += p->pp + p->pi + p->pd; 
   }
	 else *yk = +0.0; 
   v->ek_1 = ek;    																	// e[k-1] = e[k]    
	 v->lpf_2 = v->lpf_1; 															// update stores for LPF    
	 v->lpf_1 = lpf; 
	 v->yk_2 = v->yk_1;																	// y[k-2] = y[k-1]
	 v->yk_1 = *yk;																			// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM) 
    { 
     *yk = GMA_HLIM; 
    }
		 else 
		 if (*yk < GMA_LLIM) 
     { 
      *yk = GMA_LLIM; 
     } // else 
	
}

void init_tak_k1(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_tak_k1(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the Takahashi PID controller,              
which is a type C controller: the P and D term are no              
longer dependent on the set-point, only on PV (which is Thlt). 
The D term is NOT low-pass filtered. 
This function should be called once every TS seconds. 
Variables:
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
------------------------------------------------------------------*/ 
{ double_t ek;  // e[k]    
	//double_t lpf; //LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
    if (p->pid_model) 
   { 
      //----------------------------------------------------------- 
      //Calculate PID controller: 
      //y[k] = y[k-1] + Kc*(PV[k-1] - PV[k] + 
      //Ts*e[k]/Ti + 
      //Td/Ts*(2*PV[k-1] - PV[k] - PV[k-2])) 
      //-----------------------------------------------------------       
			p->pp = p->kc * (v->xk_1 - xk);										// y[k] = y[k-1] + Kc*(PV[k-1] - PV[k])       
			p->pi = p->k0 * ek;           										//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (2.0 * v->xk_1 - xk - v->xk_2); 
      *yk += p->pp + p->pi + p->pd; 
   }    else 
					{
					 *yk = p->pp = p->pi = p->pd = 0.0; 
					} 
    v->xk_2 = v->xk_1;																// PV[k-2] = PV[k-1]    
		v->xk_1 = xk;    																	// PV[k-1] = PV[k] 
		v->yk_2 = v->yk_1;																// y[k-2] = y[k-1]
		v->yk_1 = *yk;																		// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM)    
		{ 
     *yk = GMA_HLIM;    
		}    
			else 
			if (*yk < GMA_LLIM)    
			 { 
				*yk = GMA_LLIM; 
			 } // else 
	
}

void init_tak_v1(pid_params *p)
/*------------------------------------------------------------------
Purpose : This function initialises the Allen Bradley Type C PID
controller.
Variables: p: pointer to struct containing all PID parameters
Returns : No values are returned
------------------------------------------------------------------*/
/*
			 Kc.Ts
	k0 = ----- (for I-term)
				Ti
						Td
	k1 = Kc . -- (for D-term)
						Ts
The LPF parameters are also initialised here:
lpf[k] = lpf1 * lpf[k-1] + lpf2 * lpf[k-2]; lpf = p->lpf1 * lpf_1 + p->lpf2 * (ek + ek_1)
Returns : No values are returned
------------------------------------------------------------------*/
{
 if (p->ti == 0.0)
	{
	 p->k0 = 0.0;
	}
	else
	 {
		p->k0 = p->kc * p->ts / p->ti;
	 } // else
	p->k1 = p->kc * p->td / p->ts;
	p->lpf1 = (2.0 * p->k_lpf - p->ts) / (2.0 * p->k_lpf + p->ts);
	p->lpf2 = p->ts / (2.0 * p->k_lpf + p->ts);
}

void reg_tak_v1(double_t xk, double_t *yk, pid_params *p, var_params *v)
/*------------------------------------------------------------------   
Purpose  : This function implements the Takahashi PID controller,              
which is a type C controller: the P and D term are no              
longer dependent on the set-point, only on PV (which is Thlt). 
The D term is NOT low-pass filtered. 
This function should be called once every TS seconds. 
Variables:
xk : The input variable x[k] (= measured temperature) 
*yk : The output variable y[k] (= gamma value for power electronics)       
p->Soll : The setpoint value for the temperature
*p : Pointer to struct containing PID parameters 
vrg: Release signal: 1 = Start control, 0 = disable PID controller 
Returns  : No values are returned 
------------------------------------------------------------------*/ 
{ double_t ek;  // e[k]    
	//double_t lpf; //LPF output 
   ek = p->Soll - xk;  // calculate e[k] = SP[k] - PV[k] 
    if (p->pid_model) 
   { 
      //----------------------------------------------------------- 
      //Calculate PID controller: 
      //y[k] = y[k-1] + Kc*(PV[k-1] - PV[k] + 
      //Ts*e[k]/Ti + 
      //Td/Ts*(2*PV[k-1] - PV[k] - PV[k-2])) 
      //-----------------------------------------------------------       
			p->pp = p->kc * (v->xk_1 - xk);										// y[k] = y[k-1] + Kc*(PV[k-1] - PV[k])       
			p->pi = p->k0 * ek;           										//      + Kc*Ts/Ti * e[k]       
			p->pd = p->k1 * (2.0 * v->xk_1 - xk - v->xk_2); 
      *yk += p->pp + p->pi + p->pd; 
   }    else 
					{
					 *yk = p->pp = p->pi = p->pd = 0.0; 
					} 
    v->xk_2 = v->xk_1;																// PV[k-2] = PV[k-1]    
		v->xk_1 = xk;    																	// PV[k-1] = PV[k] 
		v->yk_2 = v->yk_1;																// y[k-2] = y[k-1]
		v->yk_1 = *yk;																		// y[k-1] = yk
   // limit y[k] to GMA_HLIM and GMA_LLIM    
	 if (*yk > GMA_HLIM)    
		{ 
     *yk = GMA_HLIM;    
		}    
			else 
			if (*yk < GMA_LLIM)    
			 { 
				*yk = GMA_LLIM; 
			 } // else 
	
}

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

/*------------------------------------------------------------------
Purpose : This function initialises the PID controller, based on
the new Type A? PID controller.
Variables: p: pointer to struct containing all PID parameters
   y[n] = y[n-1] + k0 * x[n] + k1 * x[n-1] + k2 * x[n-2]
   k0 = Kp + Ki + Kd
   k1 = (-Kp ) - (2 * Kd )
   k2 = Kd  
Returns : No values are returned
------------------------------------------------------------------*/

/* ARM PID Instance, double_32 format */
arm_pid_instance_f32 PID_k0;

void init_dsp_k0(pid_params *p)
{
	/* Set PID parameters */
	/* Set this for your needs */
	PID_k0.Kp = p->kc;		/* Proporcional */
	PID_k0.Ki = p->ti;		/* Integral */
	PID_k0.Kd = p->td;		/* Derivative */
	/* Initialize PID_k0 system, double32_t format */
	arm_pid_init_f32(&PID_k0,1);	//Inicializacia instancie pre PID ...
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID_k0 controller.
It is an update of pid_reg1(), It is a Type? A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID_k0 parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID_k0 controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_dsp_k0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	*yk= arm_pid_f32(&PID_k0,ek);
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
} 

/* ARM PID Instance, double_32 format */
arm_pid_instance_f32 PID_v0;

void init_dsp_v0(pid_params *p)
{
	/* Set PID parameters */
	/* Set this for your needs */
	PID_v0.Kp = p->kc;		/* Proporcional */
	PID_v0.Ki = p->ti;		/* Integral */
	PID_v0.Kd = p->td;		/* Derivative */
	/* Initialize PID_v0 system, double32_t format */
	arm_pid_init_f32(&PID_v0,1);	//Inicializacia instancie pre PID ...
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID_v0 controller.
It is an update of pid_reg1(), It is a Type? A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID_v0 parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID_v0 controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_dsp_v0(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	*yk= arm_pid_f32(&PID_v0,ek);
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
} 

/* ARM PID Instance, double_32 format */
arm_pid_instance_f32 PID_k1;

void init_dsp_k1(pid_params *p)
{
	/* Set PID parameters */
	/* Set this for your needs */
	PID_k1.Kp = p->kc;		/* Proporcional */
	PID_k1.Ki = p->ti;		/* Integral */
	PID_k1.Kd = p->td;		/* Derivative */
	/* Initialize PID_k1 system, double32_t format */
	arm_pid_init_f32(&PID_k1,1);	//Inicializacia instancie pre PID ...
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID_k1 controller.
It is an update of pid_reg1(), It is a Type? A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID_k1 parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID_k1 controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_dsp_k1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	*yk= arm_pid_f32(&PID_k1,ek);
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
} 

/* ARM PID Instance, double_32 format */
arm_pid_instance_f32 PID_v1;

void init_dsp_v1(pid_params *p)
{
	/* Set PID parameters */
	/* Set this for your needs */
	PID_v1.Kp = p->kc;		/* Proporcional */
	PID_v1.Ki = p->ti;		/* Integral */
	PID_v1.Kd = p->td;		/* Derivative */
	/* Initialize PID_v1 system, double32_t format */
	arm_pid_init_f32(&PID_v1,1);	//Inicializacia instancie pre PID ...
} 

/*------------------------------------------------------------------
Purpose : This function implements the updated PID_v1 controller.
It is an update of pid_reg1(), It is a Type? A controller.
This function should be called once every Ts seconds.
Variables:
xk 		: The input variable x[k] (= measured temperature)
*yk 	: The output variable y[k] (= gamma value for power electronics)
p->Soll 	: The setpoint value for the temperature
*p 		: Pointer to struct containing PID_v1 parameters
vrg		: Release signal: 1 = Start control, 0 = disable PID_v1 controller
Returns : No values are returned
------------------------------------------------------------------*/

void reg_dsp_v1(double_t xk, double_t *yk, pid_params *p, var_params *v)
{
	float32_t ek;																						// e[k]
	ek = p->Soll - xk; 																	// calculate e[k]
	*yk= arm_pid_f32(&PID_v1,ek);
	if (*yk > GMA_HLIM)																	// limit y[k] to GMA_HLIM and GMA_LLIM
		{
		*yk = GMA_HLIM;
		}
	else 
			if (*yk < GMA_LLIM)
				{
					*yk = GMA_LLIM;
				} // else
}



